' Extremely Simple Capture API

type SimpleCapParams
  ' Target buffer. 
  ' Must be at least mWidth * mHeight * sizeof(long) ! 
  as ulong ptr mTargetBuf
  as long      mWidth
  as long      mHeight
end type

type CAPTURE_PROPETY as ulong
const as CAPTURE_PROPETY CAPTURE_BRIGHTNESS=0
const as CAPTURE_PROPETY CAPTURE_CONTRAST=1
const as CAPTURE_PROPETY CAPTURE_HUE=2
const as CAPTURE_PROPETY CAPTURE_SATURATION=3
const as CAPTURE_PROPETY CAPTURE_SHARPNESS=4
const as CAPTURE_PROPETY CAPTURE_GAMMA=5
const as CAPTURE_PROPETY CAPTURE_COLORENABLE=6
const as CAPTURE_PROPETY CAPTURE_WHITEBALANCE=7
const as CAPTURE_PROPETY CAPTURE_BACKLIGHTCOMPENSATION=8
const as CAPTURE_PROPETY CAPTURE_GAIN=9
const as CAPTURE_PROPETY CAPTURE_PAN=10
const as CAPTURE_PROPETY CAPTURE_TILT=11
const as CAPTURE_PROPETY CAPTURE_ROLL=12
const as CAPTURE_PROPETY CAPTURE_ZOOM=13
const as CAPTURE_PROPETY CAPTURE_EXPOSURE=14
const as CAPTURE_PROPETY CAPTURE_IRIS=15
const as CAPTURE_PROPETY CAPTURE_FOCUS=16

' Sets up the ESCAPI DLL and the function pointers below. Call this first!
' Returns number of capture devices found (same as countCaptureDevices, below)
declare function setupESCAPI() as long

' return the number of capture devices found
type countCaptureDevicesProc as function cdecl as long
dim shared as countCaptureDevicesProc countCaptureDevices

' initCapture tries to open the video capture device. 
' Returns 0 on failure, 1 on success. 
' Note: Capture parameter values must not change while capture device
'       is in use (i.e. between initCapture and deinitCapture).
'       Do *not* free the target buffer, or change its pointer!
type initCaptureProc as function cdecl (deviceno as long, p as SimpleCapParams ptr) as long
dim shared as initCaptureProc initCapture

' deinitCapture closes the video capture device.
type deinitCaptureProc as sub cdecl(deviceno as long)
dim shared as deinitCaptureProc deinitCapture

' doCapture requests video frame to be captured.
type doCaptureProc as sub cdecl(deviceno as long)
dim shared as doCaptureProc doCapture

' isCaptureDone returns 1 when the requested frame has been captured.
type isCaptureDoneProc as function cdecl(deviceno as long) as long
dim shared as isCaptureDoneProc isCaptureDone

' Get the user-friendly name of a capture device.
type getCaptureDeviceNameProc as sub cdecl(deviceno as long,namebuffer as zstring ptr,bufferlength as long)
dim shared as getCaptureDeviceNameProc getCaptureDeviceName

' Returns the ESCAPI version. 0x200 for 2.0
type ESCAPIVersionProc as function() as long
dim shared as ESCAPIVersionProc ESCAPIVersion

' Returns the ESCAPI DLL version. 0x200 for 2.0
type ESCAPIDLLVersionProc as function() as long
dim shared as ESCAPIDLLVersionProc ESCAPIDLLVersion

' Internal: initialize COM
type initCOMProc as sub cdecl
dim shared as initCOMProc initCOM


' Gets value (0..1) of a camera property (see CAPTURE_PROPERTIES, above)
type getCapturePropertyValueProc as function (deviceno as long, prop as CAPTURE_PROPETY) as single
dim shared as getCapturePropertyValueProc getCapturePropertyValue
' Gets whether the property is set to automatic (see CAPTURE_PROPERTIES, above)
type getCapturePropertyAutoProc as function (deviceno as long, prop as CAPTURE_PROPETY) as long
dim shared as getCapturePropertyAutoProc getCapturePropertyAuto
' Set camera property to a value (0..1) and whether it should be set to auto.
type setCapturePropertyProc as function (deviceno as long, prop as CAPTURE_PROPETY, value as single, bAutoval as long) as long
dim shared as setCapturePropertyProc setCaptureProperty



function setupESCAPI() as long
  chdir exepath()
  ' Load DLL dynamically
#ifndef __FB_64BIT__ 
  dim as any ptr capdll = DyLibLoad("./win32/escapi.dll")
#else
  dim as any ptr capdll = DyLibLoad("./win64/escapi.dll")
#endif

  if (capdll = 0) then
    print "can't load 'escapi.dll' !"
    beep:sleep:return 0
  end if
  ' Fetch function DyLibSymbol(escapi_dll,
  countCaptureDevices     = DyLibSymbol(capdll, "countCaptureDevices")
  initCapture             = DyLibSymbol(capdll, "initCapture")
  deinitCapture           = DyLibSymbol(capdll, "deinitCapture")
  doCapture               = DyLibSymbol(capdll, "doCapture")
  isCaptureDone           = DyLibSymbol(capdll, "isCaptureDone")
  initCOM                 = DyLibSymbol(capdll, "initCOM")
  getCaptureDeviceName    = DyLibSymbol(capdll, "getCaptureDeviceName")
  ESCAPIDLLVersion        = DyLibSymbol(capdll, "ESCAPIDLLVersion")
  ESCAPIVersion           = DyLibSymbol(capdll, "ESCAPIVersion")
  getCapturePropertyValue = DyLibSymbol(capdll, "getCapturePropertyValue")
  getCapturePropertyAuto  = DyLibSymbol(capdll, "getCapturePropertyAuto")
  setCaptureProperty      = DyLibSymbol(capdll, "setCaptureProperty")


  if (countCaptureDevices=0) orelse _
     (initCapture=0) orelse _
     (deinitCapture=0) orelse _
     (doCapture=0) orelse _
     (isCaptureDone=0) orelse _
     (initCOM=0) orelse _
     (getCaptureDeviceName=0) orelse _
     (ESCAPIDLLVersion=0) orelse _
     (ESCAPIVersion=0) orelse _
     (getCapturePropertyValue=0) orelse _
     (getCapturePropertyAuto=0) orelse _
     (setCaptureProperty=0) then  
    print "can't get proc address !"
    beep: sleep: return 0
  end if
  if (ESCAPIDLLVersion() < &H200) then
    print "wrong dll version !"
    beep:sleep:return 0
  end if
  ' Initialize COM and return the number of capture devices found.
  initCOM()
  return countCaptureDevices()
end function




